-- | Display atomic commands received by the client.
module Game.LambdaHack.Client.UI.Watch
  ( watchRespUpdAtomicUI, watchRespSfxAtomicUI
  ) where

import Prelude ()

import Game.LambdaHack.Client.UI.Watch.WatchSfxAtomicM
import Game.LambdaHack.Client.UI.Watch.WatchUpdAtomicM
